import os.path
import paraview.simple as pvs

# This script flattens a multiblock dataset and applies texture and colors
# specified in field data of the multiblock.


texProxies = {}
for reader in GetSources().values():
    mb = reader.GetClientSideObject().GetOutput()
    it = mb.NewIterator()
    dirName = os.path.dirname(reader.FileName)
    baseName = os.path.basename(reader.FileName)
    Hide(reader)
    while(not it.IsDoneWithTraversal()):
        flatIndex = it.GetCurrentFlatIndex()
        extractBlock = ExtractBlock(Input=reader)
        extractBlock.BlockIndices = [flatIndex]
        extractBlockDisplay = Show(extractBlock, GetRenderView())

        dataObject = it.GetCurrentDataObject()
        texture = dataObject.GetFieldData().GetAbstractArray("texture_uri")
        diffuse = dataObject.GetFieldData().GetAbstractArray("diffuse_color")
        if (texture):
            textureURI = texture.GetValue(0)
            if (textureURI not in texProxies):
                textureFile = dirName + "/" + textureURI
                if (os.path.isfile(textureFile)):
                    # create the texture proxy
                    texProxy = pvs.servermanager.CreateProxy(
                        "textures", "ImageTexture")
                    texProxy.GetProperty("FileName").SetElement(0, textureFile)
                    texProxy.UpdateVTKObjects()
                    # ensure that the textures are saved in the state file
                    pvs.servermanager.Register(
                        texProxy, registrationName=baseName + "_" + textureURI)
                    texProxies[textureURI] = texProxy
            else:
                texProxy = texProxies[textureURI]
            # set the texture proxy
            extractBlockDisplay.Texture = texProxy
        if (diffuse):
            specular = dataObject.GetFieldData().GetAbstractArray(
                "specular_color")
            transparency = dataObject.GetFieldData().GetAbstractArray(
                "transparency")
            diffuseColor = [diffuse.GetValue(0),
                            diffuse.GetValue(1),
                            diffuse.GetValue(2)]
            extractBlockDisplay.DiffuseColor = diffuseColor
            if (specular):
                specularColor = [specular.GetValue(0),
                                 specular.GetValue(1),
                                 specular.GetValue(2)]
                extractBlockDisplay.Specular = 1.0
                extractBlockDisplay.SpecularColor = specularColor
            if (transparency):
                transparencyValue = transparency.GetValue(0)
                extractBlockDisplay.Opacity = 1 - transparencyValue
        it.GoToNextItem()
